@@ -1,9 +1,10 @@
 extern crate proc_macro;
 
+use std::{collections::HashSet, mem::replace};
+
 use inflector::Inflector;
 use pmutil::{q, Quote};
 use proc_macro2::Ident;
-use std::{collections::HashSet, mem::replace};
 use swc_macros_common::{call_site, def_site};
 use syn::{
     parse_quote::parse, punctuated::Punctuated, spanned::Spanned, Arm, AttrStyle, Attribute, Block,
@@ -94,8 +95,8 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
     // Remove `Box`
     types.retain(|ty| as_box(ty).is_none());
-    types.sort_by_cached_key(|ty| method_name_as_str(mode, &ty));
-    types.dedup_by_key(|ty| method_name_as_str(mode, &ty));
+    types.sort_by_cached_key(|ty| method_name_as_str(mode, ty));
+    types.dedup_by_key(|ty| method_name_as_str(mode, ty));
 
     let types = types;
 
@@ -106,14 +107,14 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
         let sig = create_method_sig(mode, ty);
         let name = sig.ident.clone();
         let s = name.to_string();
-        if methods.iter().any(|m| m.sig.ident == &*s) {
+        if methods.iter().any(|m| m.sig.ident == *s) {
             continue;
         }
 
         methods.push(TraitItemMethod {
             attrs: vec![],
             sig,
-            default: Some(create_method_body(mode, &ty)),
+            default: Some(create_method_body(mode, ty)),
             semi_token: None,
         });
     }
@@ -269,15 +270,13 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                     stmts: Default::default(),
                 },
             }),
-        )
-        .clone();
+        );
 
         let arg_ty = v
             .sig
             .inputs
             .iter()
-            .skip(1)
-            .next()
+            .nth(1)
             .map(|v| match *v {
                 FnArg::Typed(ref pat) => &pat.ty,
                 _ => unreachable!(),
@@ -447,7 +446,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
         let trait_decl = match mode {
             Mode::Visit => q!({
-                pub trait VisitWith<V: Visit> {
+                pub trait VisitWith<V: ?Sized + Visit> {
                     fn visit_with(&self, v: &mut V);
 
                     /// Visit children nodes of self with `v`
@@ -456,7 +455,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
                 impl<V, T> VisitWith<V> for Box<T>
                 where
-                    V: Visit,
+                    V: ?Sized + Visit,
                     T: 'static + VisitWith<V>,
                 {
                     fn visit_with(&self, v: &mut V) {
@@ -470,7 +469,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                 }
             }),
             Mode::VisitAll => q!({
-                pub trait VisitAllWith<V: VisitAll> {
+                pub trait VisitAllWith<V: ?Sized + VisitAll> {
                     fn visit_all_with(&self, v: &mut V);
 
                     /// Visit children nodes of self with `v`
@@ -479,7 +478,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
                 impl<V, T> VisitAllWith<V> for Box<T>
                 where
-                    V: VisitAll,
+                    V: ?Sized + VisitAll,
                     T: 'static + VisitAllWith<V>,
                 {
                     fn visit_all_with(&self, v: &mut V) {
@@ -493,7 +492,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                 }
             }),
             Mode::Fold => q!({
-                pub trait FoldWith<V: Fold> {
+                pub trait FoldWith<V: ?Sized + Fold> {
                     fn fold_with(self, v: &mut V) -> Self;
 
                     /// Visit children nodes of self with `v`
@@ -502,7 +501,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
                 impl<V, T> FoldWith<V> for Box<T>
                 where
-                    V: Fold,
+                    V: ?Sized + Fold,
                     T: 'static + FoldWith<V>,
                 {
                     fn fold_with(self, v: &mut V) -> Self {
@@ -516,7 +515,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                 }
             }),
             Mode::VisitMut => q!({
-                pub trait VisitMutWith<V: VisitMut> {
+                pub trait VisitMutWith<V: ?Sized + VisitMut> {
                     fn visit_mut_with(&mut self, v: &mut V);
 
                     fn visit_mut_children_with(&mut self, v: &mut V);
@@ -524,7 +523,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
 
                 impl<V, T> VisitMutWith<V> for Box<T>
                 where
-                    V: VisitMut,
+                    V: ?Sized + VisitMut,
                     T: 'static + VisitMutWith<V>,
                 {
                     fn visit_mut_with(&mut self, v: &mut V) {
@@ -580,7 +579,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                                 default_body,
                             },
                             {
-                                impl<V: Visit> VisitWith<V> for [elem_ty] {
+                                impl<V: ?Sized + Visit> VisitWith<V> for [elem_ty] {
                                     fn visit_with(&self, v: &mut V) {
                                         expr
                                     }
@@ -593,7 +592,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                         ));
 
                         tokens.push_tokens(&q!(Vars { Type: ty }, {
-                            impl<V: Visit> VisitWith<V> for Type {
+                            impl<V: ?Sized + Visit> VisitWith<V> for Type {
                                 fn visit_with(&self, v: &mut V) {
                                     (**self).visit_with(v)
                                 }
@@ -611,7 +610,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                                 default_body,
                             },
                             {
-                                impl<V: Visit> VisitWith<V> for Type {
+                                impl<V: ?Sized + Visit> VisitWith<V> for Type {
                                     fn visit_with(&self, v: &mut V) {
                                         expr
                                     }
@@ -644,7 +643,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                             default_body,
                         },
                         {
-                            impl<V: VisitAll> VisitAllWith<V> for Type {
+                            impl<V: ?Sized + VisitAll> VisitAllWith<V> for Type {
                                 fn visit_all_with(&self, v: &mut V) {
                                     let mut all = ::swc_visit::All { visitor: v };
                                     let mut v = &mut all;
@@ -680,7 +679,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                             expr,
                         },
                         {
-                            impl<V: VisitMut> VisitMutWith<V> for Type {
+                            impl<V: ?Sized + VisitMut> VisitMutWith<V> for Type {
                                 fn visit_mut_with(&mut self, v: &mut V) {
                                     expr
                                 }
@@ -701,7 +700,7 @@ fn make(mode: Mode, stmts: &[Stmt]) -> Quote {
                             expr,
                         },
                         {
-                            impl<V: Fold> FoldWith<V> for Type {
+                            impl<V: ?Sized + Fold> FoldWith<V> for Type {
                                 fn fold_with(self, v: &mut V) -> Self {
                                     expr
                                 }
@@ -724,7 +723,7 @@ fn adjust_expr<F>(mode: Mode, ty: &Type, mut expr: Expr, visit: F) -> Expr
 where
     F: FnOnce(Expr) -> Expr,
 {
-    if is_option(&ty) {
+    if is_option(ty) {
         expr = if is_opt_vec(ty) {
             match mode {
                 Mode::Fold => expr,
@@ -1099,25 +1098,25 @@ fn create_method_sig(mode: Mode, ty: &Type) -> Signature {
         Type::Infer(_) => unreachable!("infer type"),
         Type::Macro(_) => unimplemented!("type: macro"),
         Type::Never(_) => unreachable!("never type"),
-        Type::Paren(ty) => return create_method_sig(mode, &ty.elem),
+        Type::Paren(ty) => create_method_sig(mode, &ty.elem),
         Type::Path(p) => {
             let last = p.path.segments.last().unwrap();
             let ident = method_name(mode, ty);
 
             if !last.arguments.is_empty() {
-                if let Some(arg) = as_box(&ty) {
-                    let ident = method_name(mode, &arg);
+                if let Some(arg) = as_box(ty) {
+                    let ident = method_name(mode, arg);
                     match mode {
                         Mode::Fold => {
-                            return mk_exact(mode, ident, &arg);
+                            return mk_exact(mode, ident, arg);
                         }
 
                         Mode::VisitMut => {
-                            return mk_ref(mode, ident, &arg, true);
+                            return mk_ref(mode, ident, arg, true);
                         }
 
                         Mode::Visit | Mode::VisitAll => {
-                            return mk_ref(mode, ident, &arg, false);
+                            return mk_ref(mode, ident, arg, false);
                         }
                     }
                 }
@@ -1220,19 +1219,13 @@ fn create_method_sig(mode: Mode, ty: &Type) -> Signature {
             }
 
             match mode {
-                Mode::Fold => return mk_exact(mode, ident, ty),
-                Mode::VisitMut => {
-                    return mk_ref(mode, ident, ty, true);
-                }
-                Mode::Visit | Mode::VisitAll => {
-                    return mk_ref(mode, ident, ty, false);
-                }
+                Mode::Fold => mk_exact(mode, ident, ty),
+                Mode::VisitMut => mk_ref(mode, ident, ty, true),
+                Mode::Visit | Mode::VisitAll => mk_ref(mode, ident, ty, false),
             }
         }
         Type::Ptr(_) => unimplemented!("type: pointer"),
-        Type::Reference(ty) => {
-            return create_method_sig(mode, &ty.elem);
-        }
+        Type::Reference(ty) => create_method_sig(mode, &ty.elem),
         Type::Slice(_) => unimplemented!("type: slice"),
         Type::TraitObject(_) => unimplemented!("type: trait object"),
         Type::Tuple(_) => unimplemented!("type: trait tuple"),
@@ -1267,7 +1260,7 @@ fn create_method_body(mode: Mode, ty: &Type) -> Block {
         Type::Infer(_) => unreachable!("infer type"),
         Type::Macro(_) => unimplemented!("type: macro"),
         Type::Never(_) => unreachable!("never type"),
-        Type::Paren(ty) => return create_method_body(mode, &ty.elem),
+        Type::Paren(ty) => create_method_body(mode, &ty.elem),
         Type::Path(p) => {
             let last = p.path.segments.last().unwrap();
 
@@ -1298,27 +1291,24 @@ fn create_method_body(mode: Mode, ty: &Type) -> Block {
                                 GenericArgument::Type(arg) => {
                                     let ident = method_name(mode, arg);
 
-                                    match mode {
-                                        Mode::Fold => {
-                                            if let Some(..) = as_box(arg) {
-                                                return q!(
-                                                    Vars { ident },
-                                                    ({
-                                                        match n {
-                                                            Some(n) => Some(
-                                                                swc_visit::util::map::Map::map(
-                                                                    n,
-                                                                    |n| _visitor.ident(n),
-                                                                ),
-                                                            ),
-                                                            None => None,
+                                    if mode == Mode::Fold {
+                                        if let Some(..) = as_box(arg) {
+                                            return q!(
+                                                Vars { ident },
+                                                ({
+                                                    match n {
+                                                        Some(n) => {
+                                                            Some(swc_visit::util::map::Map::map(
+                                                                n,
+                                                                |n| _visitor.ident(n),
+                                                            ))
                                                         }
-                                                    })
-                                                )
-                                                .parse();
-                                            }
+                                                        None => None,
+                                                    }
+                                                })
+                                            )
+                                            .parse();
                                         }
-                                        _ => {}
                                     }
 
                                     return match mode {
@@ -1458,9 +1448,7 @@ fn create_method_body(mode: Mode, ty: &Type) -> Block {
             }
         }
         Type::Ptr(_) => unimplemented!("type: pointer"),
-        Type::Reference(ty) => {
-            return create_method_body(mode, &ty.elem);
-        }
+        Type::Reference(ty) => create_method_body(mode, &ty.elem),
         Type::Slice(_) => unimplemented!("type: slice"),
         Type::TraitObject(_) => unimplemented!("type: trait object"),
         Type::Tuple(_) => unimplemented!("type: trait tuple"),
@@ -1483,22 +1471,16 @@ fn add_required(types: &mut Vec<Type>, ty: &Type) {
     if let Some(ty) = extract_generic("Arc", ty) {
         add_required(types, ty);
         types.push(ty.clone());
-        return;
     }
 }
 
 fn is_option(ty: &Type) -> bool {
-    match ty {
-        Type::Path(p) => {
-            let last = p.path.segments.last().unwrap();
+    if let Type::Path(p) = ty {
+        let last = p.path.segments.last().unwrap();
 
-            if !last.arguments.is_empty() {
-                if last.ident == "Option" {
-                    return true;
-                }
-            }
+        if !last.arguments.is_empty() && last.ident == "Option" {
+            return true;
         }
-        _ => {}
     }
 
     false
@@ -1509,27 +1491,22 @@ fn as_box(ty: &Type) -> Option<&Type> {
 }
 
 fn extract_generic<'a>(name: &str, ty: &'a Type) -> Option<&'a Type> {
-    match ty {
-        Type::Path(p) => {
-            let last = p.path.segments.last().unwrap();
+    if let Type::Path(p) = ty {
+        let last = p.path.segments.last().unwrap();
 
-            if !last.arguments.is_empty() {
-                if last.ident == name {
-                    match &last.arguments {
-                        PathArguments::AngleBracketed(tps) => {
-                            let arg = tps.args.first().unwrap();
+        if !last.arguments.is_empty() && last.ident == name {
+            match &last.arguments {
+                PathArguments::AngleBracketed(tps) => {
+                    let arg = tps.args.first().unwrap();
 
-                            match arg {
-                                GenericArgument::Type(arg) => return Some(arg),
-                                _ => unimplemented!("generic parameter other than type"),
-                            }
-                        }
-                        _ => unimplemented!("Box() -> T or Box without a type parameter"),
+                    match arg {
+                        GenericArgument::Type(arg) => return Some(arg),
+                        _ => unimplemented!("generic parameter other than type"),
                     }
                 }
+                _ => unimplemented!("Box() -> T or Box without a type parameter"),
             }
         }
-        _ => {}
     }
 
     None
@@ -1567,9 +1544,9 @@ fn method_name_as_str(mode: Mode, ty: &Type) -> String {
             if suffix(ty).to_plural() == suffix(ty) {
                 return format!("{}_vec", suffix(ty).to_plural());
             }
-            return format!("{}", suffix(ty).to_plural());
+            return suffix(ty).to_plural();
         }
-        type_to_name(&ty).to_snake_case()
+        type_to_name(ty).to_snake_case()
     }
 
     format!("{}_{}", mode.prefix(), suffix(ty))
@@ -1592,8 +1569,7 @@ fn skip(ty: &Type) -> bool {
         Type::Path(p) => {
             let i = &p.path.segments.last().as_ref().unwrap().ident;
 
-            if i == "bool"
-                || i == "u128"
+            p.path.segments.last().as_ref().unwrap().ident == "bool"
                 || i == "u128"
                 || i == "u64"
                 || i == "u32"
@@ -1601,19 +1577,6 @@ fn skip(ty: &Type) -> bool {
                 || i == "u8"
                 || i == "isize"
                 || i == "i128"
-                || i == "i128"
-                || i == "i64"
-                || i == "i32"
-                || i == "i16"
-                || i == "i8"
-                || i == "isize"
-                || i == "f64"
-                || i == "f32"
-            {
-                return true;
-            }
-
-            false
         }
         _ => false,
     }
